@@ -21,14 +21,15 @@ Create Date: 2015-02-02 13:11:55.184112
 
 """
 
-from alembic import op
-import sqlalchemy as sa
-
 # revision identifiers, used by Alembic.
 revision = '796c68dffbb'
 down_revision = '540142f314f4'
 
 
+from alembic import op
+import sqlalchemy as sa
+
+
 def upgrade(active_plugins=None, options=None):
 
     op.create_table('cisco_firewall_associations',
@@ -43,3 +44,8 @@ def upgrade(active_plugins=None, options=None):
             ondelete='CASCADE'),
         sa.PrimaryKeyConstraint('fw_id')
     )
+
+
+def downgrade(active_plugins=None, options=None):
+
+    op.drop_table('cisco_firewall_associations')
