@@ -13,25 +13,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron.common import rpc as n_rpc
-from neutron_lib.agent import l3_extension
-from neutron_lib import constants as nl_constants
-from neutron_lib import context
-from neutron_lib.exceptions import firewall_v1 as fw_ext
+from neutron.agent.linux import ip_lib
+from neutron.common import log
+from neutron.common import topics
+from neutron import context
+from neutron.i18n import _LE
+from neutron.plugins.common import constants
 from oslo_config import cfg
-from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 
-from neutron_fwaas.common import fwaas_constants
-from neutron_fwaas.common import resources as f_resources
+from neutron_fwaas.extensions import firewall as fw_ext
 from neutron_fwaas.services.firewall.agents import firewall_agent_api as api
 from neutron_fwaas.services.firewall.agents import firewall_service
 
 LOG = logging.getLogger(__name__)
 
-#TODO(njohnston): There needs to be some extrapolation of the common code
-# between this module and firewall_l3_agent_v2.py.
-
 
 class FWaaSL3PluginApi(api.FWaaSPluginApiMixin):
     """Agent side of the FWaaS agent to FWaaS Plugin RPC API."""
@@ -52,48 +48,22 @@ class FWaaSL3PluginApi(api.FWaaSPluginApiMixin):
                           'get_tenants_with_firewalls', host=self.host)
 
 
-class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
+class FWaaSL3AgentRpcCallback(api.FWaaSAgentRpcCallbackMixin):
     """FWaaS Agent support to be used by Neutron L3 agent."""
 
-    SUPPORTED_RESOURCE_TYPES = [f_resources.FIREWALL_GROUP,
-                                f_resources.FIREWALL_POLICY,
-                                f_resources.FIREWALL_RULE]
-
-    def initialize(self, connection, driver_type):
-        self._register_rpc_consumers(connection)
-
-    def consume_api(self, agent_api):
-        LOG.debug("FWaaS consume_api call occurred with %s", agent_api)
-        self.agent_api = agent_api
-
-    def _register_rpc_consumers(self, connection):
-        #TODO(njohnston): Add RPC consumer connection loading here.
-        pass
-
-    def start_rpc_listeners(self, conf):
-        self.endpoints = [self]
-
-        self.conn = n_rpc.create_connection()
-        self.conn.create_consumer(
-            fwaas_constants.FW_AGENT, self.endpoints, fanout=False)
-        return self.conn.consume_in_threads()
-
-    def __init__(self, host, conf):
+    def __init__(self, conf):
         LOG.debug("Initializing firewall agent")
-        self.agent_api = None
-        self.neutron_service_plugins = None
         self.conf = conf
         self.fwaas_enabled = cfg.CONF.fwaas.enabled
-        self.start_rpc_listeners(conf)
 
         # None means l3-agent has no information on the server
         # configuration due to the lack of RPC support.
         if self.neutron_service_plugins is not None:
-            fwaas_plugin_configured = (fwaas_constants.FIREWALL
+            fwaas_plugin_configured = (constants.FIREWALL
                                        in self.neutron_service_plugins)
             if fwaas_plugin_configured and not self.fwaas_enabled:
-                msg = ("FWaaS plugin is configured in the server side, but "
-                       "FWaaS is disabled in L3-agent.")
+                msg = _("FWaaS plugin is configured in the server side, but "
+                        "FWaaS is disabled in L3-agent.")
                 LOG.error(msg)
                 raise SystemExit(1)
             self.fwaas_enabled = self.fwaas_enabled and fwaas_plugin_configured
@@ -104,8 +74,9 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             self.fwaas_driver = self.fw_service.load_device_drivers()
         self.services_sync_needed = False
         # setup RPC to msg fwaas plugin
-        self.fwplugin_rpc = FWaaSL3PluginApi(fwaas_constants.FIREWALL_PLUGIN,
-                                             host)
+        self.fwplugin_rpc = FWaaSL3PluginApi(topics.FIREWALL_PLUGIN,
+                                             conf.host)
+        super(FWaaSL3AgentRpcCallback, self).__init__(host=conf.host)
 
     def _has_router_insertion_fields(self, fw):
         return 'add-router-ids' in fw
@@ -117,26 +88,47 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             return (fw['del-router-ids'] if to_delete
                     else fw['add-router-ids'])
         else:
-            return [router['id'] for router in
-                self.agent_api.get_routers_in_project(fw['tenant_id'])]
-
-    def _get_routers_in_project(self, project_id):
-        if self.agent_api is None:
-            LOG.exception("FWaaS RPC call failed; L3 agent_api failure")
-        return self.agent_api.get_routers_in_project(project_id)
+            # we are in a upgrade and msg from older version of plugin
+            try:
+                routers = self.plugin_rpc.get_routers(context)
+            except Exception:
+                LOG.exception(
+                    _LE("FWaaS RPC failure in _get_router_ids_for_fw "
+                        "for firewall: %(fwid)s"),
+                    {'fwid': fw['id']})
+                self.services_sync_needed = True
+            return [
+                router['id']
+                for router in routers
+                if router['tenant_id'] == fw['tenant_id']]
 
     def _get_router_info_list_for_tenant(self, router_ids, tenant_id):
         """Returns the list of router info objects on which to apply the fw."""
-        return [ri for ri in self._get_routers_in_project(tenant_id)
-                if ri.router_id in router_ids and
-                self.agent_api.is_router_in_namespace(ri.router_id)]
+        root_ip = ip_lib.IPWrapper()
+        local_ns_list = (root_ip.get_namespaces()
+                         if self.conf.use_namespaces else [])
+
+        router_info_list = []
+        # Pick up namespaces for Tenant Routers
+        for rid in router_ids:
+            # for routers without an interface - get_routers returns
+            # the router - but this is not yet populated in router_info
+            if rid not in self.router_info:
+                continue
+            if self.conf.use_namespaces:
+                router_ns = self.router_info[rid].ns_name
+                if router_ns in local_ns_list:
+                    router_info_list.append(self.router_info[rid])
+            else:
+                router_info_list.append(self.router_info[rid])
+        return router_info_list
 
     def _invoke_driver_for_sync_from_plugin(self, ctx, router_info_list, fw):
         """Invoke the delete driver method for status of PENDING_DELETE and
         update method for all other status to (re)apply on driver which is
         Idempotent.
         """
-        if fw['status'] == nl_constants.PENDING_DELETE:
+        if fw['status'] == constants.PENDING_DELETE:
             try:
                 self.fwaas_driver.delete_firewall(
                     self.conf.agent_mode,
@@ -146,13 +138,13 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                     ctx,
                     fw['id'])
             except fw_ext.FirewallInternalDriverError:
-                LOG.error("Firewall Driver Error on fw state %(fwmsg)s "
-                          "for fw: %(fwid)s",
+                LOG.error(_LE("Firewall Driver Error on fw state %(fwmsg)s "
+                              "for fw: %(fwid)s"),
                           {'fwmsg': fw['status'], 'fwid': fw['id']})
                 self.fwplugin_rpc.set_firewall_status(
                     ctx,
                     fw['id'],
-                    nl_constants.ERROR)
+                    constants.ERROR)
         else:
             # PENDING_UPDATE, PENDING_CREATE, ...
             try:
@@ -161,31 +153,31 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                     router_info_list,
                     fw)
                 if fw['admin_state_up']:
-                    status = nl_constants.ACTIVE
+                    status = constants.ACTIVE
                 else:
-                    status = nl_constants.DOWN
+                    status = constants.DOWN
             except fw_ext.FirewallInternalDriverError:
-                LOG.error("Firewall Driver Error on fw state %(fwmsg)s "
-                          "for fw: %(fwid)s",
+                LOG.error(_LE("Firewall Driver Error on fw state %(fwmsg)s "
+                              "for fw: %(fwid)s"),
                           {'fwmsg': fw['status'], 'fwid': fw['id']})
-                status = nl_constants.ERROR
+                status = constants.ERROR
 
             self.fwplugin_rpc.set_firewall_status(
                 ctx,
                 fw['id'],
                 status)
 
-    def _process_router_add(self, router):
+    def _process_router_add(self, ri):
         """On router add, get fw with rules from plugin and update driver."""
-        LOG.debug("Process router add, router_id: '%s'", router['id'])
-        router_ids = router['id']
+        LOG.debug("Process router add, router_id: '%s'", ri.router['id'])
+        router_ids = ri.router['id']
         router_info_list = self._get_router_info_list_for_tenant(
             [router_ids],
-            router['tenant_id'])
+            ri.router['tenant_id'])
         if router_info_list:
             # Get the firewall with rules
             # for the tenant the router is on.
-            ctx = context.Context('', router['tenant_id'])
+            ctx = context.Context('', ri.router['tenant_id'])
             fw_list = self.fwplugin_rpc.get_firewalls_for_tenant(ctx)
             for fw in fw_list:
                 if self._has_router_insertion_fields(fw):
@@ -200,7 +192,7 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 # router can be present only on one fw
                 return
 
-    def add_router(self, context, new_router):
+    def process_router_add(self, ri):
         """On router add, get fw with rules from plugin and update driver.
 
         Handles agent restart, when a router is added, query the plugin to
@@ -211,25 +203,15 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         if not self.fwaas_enabled:
             return
         try:
-            self._process_router_add(new_router)
+            # TODO(sridar): as per discussion with pc_m, we may want to hook
+            # this up to the l3 agent observer notification
+            self._process_router_add(ri)
         except Exception:
             LOG.exception(
-                "FWaaS RPC info call failed for '%s'.", new_router['id'])
+                _LE("FWaaS RPC info call failed for '%s'."),
+                ri.router['id'])
             self.services_sync_needed = True
 
-    def update_router(self, context, updated_router):
-        """The update_router method is just a synonym for add_router"""
-        self.add_router(context, updated_router)
-
-    def delete_router(self, context, new_router):
-        """Handles router deletion. There is basically nothing to do for this
-        in the context of FWaaS with an IPTables driver; the namespace will
-        already have been deleted, taking the IPTables rules with it.
-        """
-        #TODO(njohnston): When another firewall driver is implemented, look at
-        # expanding this out so that the driver can handle deletion calls.
-        pass
-
     def process_services_sync(self, ctx):
         if not self.services_sync_needed:
             return
@@ -247,17 +229,17 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 ctx = context.Context('', tenant_id)
                 fw_list = self.fwplugin_rpc.get_firewalls_for_tenant(ctx)
                 for fw in fw_list:
-                    if fw['status'] == nl_constants.PENDING_DELETE:
+                    if fw['status'] == constants.PENDING_DELETE:
                         self.delete_firewall(ctx, fw, self.host)
                     # no need to apply sync data for ACTIVE fw
-                    elif fw['status'] != nl_constants.ACTIVE:
+                    elif fw['status'] != constants.ACTIVE:
                         self.update_firewall(ctx, fw, self.host)
             self.services_sync_needed = False
         except Exception:
-            LOG.exception("Failed fwaas process services sync")
+            LOG.exception(_LE("Failed fwaas process services sync"))
             self.services_sync_needed = True
 
-    @log_helpers.log_method_call
+    @log.log
     def create_firewall(self, context, firewall, host):
         """Handle Rpc from plugin to create a firewall."""
 
@@ -276,13 +258,14 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 router_info_list,
                 firewall)
             if firewall['admin_state_up']:
-                status = nl_constants.ACTIVE
+                status = constants.ACTIVE
             else:
-                status = nl_constants.DOWN
+                status = constants.DOWN
         except fw_ext.FirewallInternalDriverError:
-            LOG.error("Firewall Driver Error for create_firewall "
-                      "for firewall: %s", firewall['id'])
-            status = nl_constants.ERROR
+            LOG.error(_LE("Firewall Driver Error for create_firewall "
+                          "for firewall: %(fwid)s"),
+                {'fwid': firewall['id']})
+            status = constants.ERROR
 
         try:
             # send status back to plugin
@@ -291,11 +274,13 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 firewall['id'],
                 status)
         except Exception:
-            LOG.exception("FWaaS RPC failure in create_firewall "
-                          "for firewall: %s", firewall['id'])
+            LOG.exception(
+                _LE("FWaaS RPC failure in create_firewall "
+                    "for firewall: %(fwid)s"),
+                {'fwid': firewall['id']})
             self.services_sync_needed = True
 
-    @log_helpers.log_method_call
+    @log.log
     def update_firewall(self, context, firewall, host):
         """Handle Rpc from plugin to update a firewall."""
 
@@ -320,24 +305,24 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                         router_info_list,
                         firewall)
                     if firewall['last-router']:
-                        status = nl_constants.INACTIVE
+                        status = constants.INACTIVE
                     elif firewall['admin_state_up']:
-                        status = nl_constants.ACTIVE
+                        status = constants.ACTIVE
                     else:
-                        status = nl_constants.DOWN
+                        status = constants.DOWN
                 except fw_ext.FirewallInternalDriverError:
-                    LOG.error(
-                        "Firewall Driver Error for "
-                        "update_firewall for firewall: %s", firewall['id'])
-                    status = nl_constants.ERROR
+                    LOG.error(_LE("Firewall Driver Error for "
+                                  "update_firewall for firewall: "
+                                  "%(fwid)s"),
+                        {'fwid': firewall['id']})
+                    status = constants.ERROR
 
-        # handle the add router and/or rule, policy, firewall
-        # attribute updates
-        if status not in (nl_constants.ERROR, nl_constants.INACTIVE):
+            # the add
+        if status not in (constants.ERROR, constants.INACTIVE):
             router_ids = self._get_router_ids_for_fw(context, firewall)
-            if router_ids or firewall['router_ids']:
+            if router_ids:
                 router_info_list = self._get_router_info_list_for_tenant(
-                    router_ids + firewall['router_ids'],
+                    router_ids,
                     firewall['tenant_id'])
                 LOG.debug("Update: Add firewall on Router List: '%s'",
                     [ri.router['id'] for ri in router_info_list])
@@ -348,16 +333,16 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                         router_info_list,
                         firewall)
                     if firewall['admin_state_up']:
-                        status = nl_constants.ACTIVE
+                        status = constants.ACTIVE
                     else:
-                        status = nl_constants.DOWN
+                        status = constants.DOWN
                 except fw_ext.FirewallInternalDriverError:
-                    LOG.error(
-                        "Firewall Driver Error for "
-                        "update_firewall for firewall: %s", firewall['id'])
-                    status = nl_constants.ERROR
-            else:
-                status = nl_constants.INACTIVE
+                    LOG.error(_LE("Firewall Driver Error for "
+                                  "update_firewall for firewall: "
+                                  "%(fwid)s"),
+                        {'fwid': firewall['id']})
+                    status = constants.ERROR
+
         try:
             # send status back to plugin
             self.fwplugin_rpc.set_firewall_status(
@@ -365,11 +350,13 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 firewall['id'],
                 status)
         except Exception:
-            LOG.exception("FWaaS RPC failure in update_firewall "
-                          "for firewall: %s", firewall['id'])
+            LOG.exception(
+                _LE("FWaaS RPC failure in update_firewall "
+                    "for firewall: %(fwid)s"),
+                {'fwid': firewall['id']})
             self.services_sync_needed = True
 
-    @log_helpers.log_method_call
+    @log.log
     def delete_firewall(self, context, firewall, host):
         """Handle Rpc from plugin to delete a firewall."""
 
@@ -379,10 +366,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             router_info_list = self._get_router_info_list_for_tenant(
                 router_ids,
                 firewall['tenant_id'])
-            LOG.debug(
-                "Delete firewall %(fw)s on routers: '%(routers)s'",
-                {'fw': firewall['id'],
-                 'routers': [ri.router['id'] for ri in router_info_list]})
+            LOG.debug("Delete: Delete firewall on Router List: '%s'",
+                [ri.router['id'] for ri in router_info_list])
             # call into the driver
             try:
                 self.fwaas_driver.delete_firewall(
@@ -390,17 +375,18 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                     router_info_list,
                     firewall)
                 if firewall['admin_state_up']:
-                    status = nl_constants.ACTIVE
+                    status = constants.ACTIVE
                 else:
-                    status = nl_constants.DOWN
+                    status = constants.DOWN
             except fw_ext.FirewallInternalDriverError:
-                LOG.error("Firewall Driver Error for delete_firewall "
-                          "for firewall: %s", firewall['id'])
-                status = nl_constants.ERROR
+                LOG.error(_LE("Firewall Driver Error for delete_firewall "
+                              "for firewall: %(fwid)s"),
+                    {'fwid': firewall['id']})
+                status = constants.ERROR
 
             try:
                 # send status back to plugin
-                if status in [nl_constants.ACTIVE, nl_constants.DOWN]:
+                if status in [constants.ACTIVE, constants.DOWN]:
                     self.fwplugin_rpc.firewall_deleted(context, firewall['id'])
                 else:
                     self.fwplugin_rpc.set_firewall_status(
@@ -408,19 +394,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                         firewall['id'],
                         status)
             except Exception:
-                LOG.exception("FWaaS RPC failure in delete_firewall "
-                              "for firewall: %s", firewall['id'])
+                LOG.exception(
+                    _LE("FWaaS RPC failure in delete_firewall "
+                        "for firewall: %(fwid)s"),
+                    {'fwid': firewall['id']})
                 self.services_sync_needed = True
-
-    def ha_state_change(self, context, data):
-        pass
-
-
-class L3WithFWaaS(FWaaSL3AgentExtension):
-
-    def __init__(self, conf=None):
-        if conf:
-            self.conf = conf
-        else:
-            self.conf = cfg.CONF
-        super(L3WithFWaaS, self).__init__(host=self.conf.host, conf=self.conf)
