@@ -17,9 +17,9 @@ import requests
 
 from networking_cisco.plugins.cisco.cfg_agent.device_drivers import (
     cisco_csr_rest_client)
+from neutron.i18n import _LE
 from oslo_log import log as logging
 
-from neutron_fwaas._i18n import _LE
 from neutron_fwaas.services.firewall.drivers import fwaas_base
 
 
@@ -144,10 +144,7 @@ class CsrAclDriver(fwaas_base.FwaasDriverBase):
             ace_rule = {'sequence': str(seq)}
             seq += 1
 
-            if rule.get('protocol'):
-                ace_rule['protocol'] = rule['protocol']
-            else:
-                ace_rule['protocol'] = 'all'
+            ace_rule['protocol'] = rule['protocol']
 
             if rule['action'].lower() == 'allow':
                 ace_rule['action'] = 'permit'
@@ -168,7 +165,7 @@ class CsrAclDriver(fwaas_base.FwaasDriverBase):
             self._get_acl_l4_port('source_port', rule, l4_opt)
             self._get_acl_l4_port('destination_port', rule, l4_opt)
             if l4_opt:
-                ace_rule['l4-options'] = l4_opt
+                ace_rule['L4-options'] = l4_opt
 
             acl_rules_list.append(ace_rule)
 
@@ -300,16 +297,16 @@ class CsrAclDriver(fwaas_base.FwaasDriverBase):
 
         acl_id = firewall['vendor_ext'].get('acl_id')
         if not acl_id:
-            LOG.error(_LE("firewall (%s) has no acl_id"), firewall['id'])
+            LOG.error(_LE("firewal (%s) has no acl_id"), firewall['id'])
             return False
 
         csr = self._get_csr_host(firewall['vendor_ext'])
         return self._delete_acl(csr, acl_id)
 
     def _update_firewall(self, firewall):
-        """Update ACL and associated interfaces.
+        """Update ACL and associated interfacesr.
 
-        :param firewall: firewall dictionary
+        :param firewall: firewall dictionry
         :return: True and status_data if OK
         :return: False and {} if there is an error
         """
@@ -321,7 +318,7 @@ class CsrAclDriver(fwaas_base.FwaasDriverBase):
 
         acl_id = firewall['vendor_ext'].get('acl_id')
         if not acl_id:
-            LOG.error(_LE("firewall (%s) has no acl_id"), firewall['id'])
+            LOG.error(_LE("firewal (%s) has no acl_id"), firewall['id'])
             return False, {}
 
         csr = self._get_csr_host(firewall['vendor_ext'])
