@@ -14,33 +14,26 @@
 #    under the License.
 
 from neutron.agent.linux import iptables_manager
-from neutron.common import utils
-from neutron_lib import constants
-from neutron_lib.exceptions import firewall_v1 as f_exc
+from neutron.i18n import _LE
 from oslo_log import log as logging
 
-from neutron_fwaas.common import fwaas_constants as f_const
-from neutron_fwaas.services.firewall.drivers import conntrack_base
+from neutron_fwaas.extensions import firewall as fw_ext
 from neutron_fwaas.services.firewall.drivers import fwaas_base
 
-
 LOG = logging.getLogger(__name__)
 FWAAS_DRIVER_NAME = 'Fwaas iptables driver'
 FWAAS_DEFAULT_CHAIN = 'fwaas-default-policy'
-
-FWAAS_TO_IPTABLE_ACTION_MAP = {'allow': 'ACCEPT',
-                               'deny': 'DROP',
-                               'reject': 'REJECT'}
-
-CHAIN_NAME_PREFIX = {constants.INGRESS_DIRECTION: 'i',
-                     constants.EGRESS_DIRECTION: 'o'}
+INGRESS_DIRECTION = 'ingress'
+EGRESS_DIRECTION = 'egress'
+CHAIN_NAME_PREFIX = {INGRESS_DIRECTION: 'i',
+                     EGRESS_DIRECTION: 'o'}
 
 """ Firewall rules are applied on internal-interfaces of Neutron router.
     The packets ingressing tenant's network will be on the output
     direction on internal-interfaces.
 """
-IPTABLES_DIR = {constants.INGRESS_DIRECTION: '-o',
-                constants.EGRESS_DIRECTION: '-i'}
+IPTABLES_DIR = {INGRESS_DIRECTION: '-o',
+                EGRESS_DIRECTION: '-i'}
 IPV4 = 'ipv4'
 IPV6 = 'ipv6'
 IP_VER_TAG = {IPV4: 'v4',
@@ -56,24 +49,19 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
 
     def __init__(self):
         LOG.debug("Initializing fwaas iptables driver")
-        self.pre_firewall = None
-        self.conntrack = conntrack_base.load_and_init_conntrack_driver()
 
     def create_firewall(self, agent_mode, apply_list, firewall):
-        LOG.debug('Creating firewall %(fw_id)s for tenant %(tid)s',
+        LOG.debug('Creating firewall %(fw_id)s for tenant %(tid)s)',
                   {'fw_id': firewall['id'], 'tid': firewall['tenant_id']})
         try:
             if firewall['admin_state_up']:
                 self._setup_firewall(agent_mode, apply_list, firewall)
-                self._remove_conntrack_new_firewall(agent_mode,
-                                                    apply_list, firewall)
-                self.pre_firewall = dict(firewall)
             else:
                 self.apply_default_policy(agent_mode, apply_list, firewall)
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception("Failed to create firewall: %s", firewall['id'])
-            raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
+            LOG.exception(_LE("Failed to create firewall: %s"), firewall['id'])
+            raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def _get_ipt_mgrs_with_if_prefix(self, agent_mode, router_info):
         """Gets the iptables manager along with the if prefix to apply rules.
@@ -94,14 +82,14 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
             if router_info.snat_iptables_manager:
                 ipt_mgrs.append({'ipt': router_info.snat_iptables_manager,
                                  'if_prefix': SNAT_INT_DEV_PREFIX})
-        if router_info.rtr_fip_connect:
+        if router_info.dist_fip_count:
             # handle the fip case on n/w or compute node.
             ipt_mgrs.append({'ipt': router_info.iptables_manager,
                              'if_prefix': ROUTER_2_FIP_DEV_PREFIX})
         return ipt_mgrs
 
     def delete_firewall(self, agent_mode, apply_list, firewall):
-        LOG.debug('Deleting firewall %(fw_id)s for tenant %(tid)s',
+        LOG.debug('Deleting firewall %(fw_id)s for tenant %(tid)s)',
                   {'fw_id': firewall['id'], 'tid': firewall['tenant_id']})
         fwid = firewall['id']
         try:
@@ -114,34 +102,26 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
                     self._remove_default_chains(ipt_mgr)
                     # apply the changes immediately (no defer in firewall path)
                     ipt_mgr.defer_apply_off()
-            self.pre_firewall = None
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception("Failed to delete firewall: %s", fwid)
-            raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
+            LOG.exception(_LE("Failed to delete firewall: %s"), fwid)
+            raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def update_firewall(self, agent_mode, apply_list, firewall):
-        LOG.debug('Updating firewall %(fw_id)s for tenant %(tid)s',
+        LOG.debug('Updating firewall %(fw_id)s for tenant %(tid)s)',
                   {'fw_id': firewall['id'], 'tid': firewall['tenant_id']})
         try:
             if firewall['admin_state_up']:
                 self._setup_firewall(agent_mode, apply_list, firewall)
-                if self.pre_firewall:
-                    self._remove_conntrack_updated_firewall(agent_mode,
-                                    apply_list, self.pre_firewall, firewall)
-                else:
-                    self._remove_conntrack_new_firewall(agent_mode,
-                                                    apply_list, firewall)
             else:
                 self.apply_default_policy(agent_mode, apply_list, firewall)
-            self.pre_firewall = dict(firewall)
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception("Failed to update firewall: %s", firewall['id'])
-            raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
+            LOG.exception(_LE("Failed to update firewall: %s"), firewall['id'])
+            raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def apply_default_policy(self, agent_mode, apply_list, firewall):
-        LOG.debug('Applying firewall %(fw_id)s for tenant %(tid)s',
+        LOG.debug('Applying firewall %(fw_id)s for tenant %(tid)s)',
                   {'fw_id': firewall['id'], 'tid': firewall['tenant_id']})
         fwid = firewall['id']
         try:
@@ -163,8 +143,8 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
             LOG.exception(
-                "Failed to apply default policy on firewall: %s", fwid)
-            raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
+                _LE("Failed to apply default policy on firewall: %s"), fwid)
+            raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def _setup_firewall(self, agent_mode, apply_list, firewall):
         fwid = firewall['id']
@@ -206,10 +186,8 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
                 table = ipt_mgr.ipv4['filter']
             else:
                 table = ipt_mgr.ipv6['filter']
-            ichain_name = self._get_chain_name(
-                fwid, ver, constants.INGRESS_DIRECTION)
-            ochain_name = self._get_chain_name(
-                fwid, ver, constants.EGRESS_DIRECTION)
+            ichain_name = self._get_chain_name(fwid, ver, INGRESS_DIRECTION)
+            ochain_name = self._get_chain_name(fwid, ver, EGRESS_DIRECTION)
             for name in [ichain_name, ochain_name]:
                 table.add_chain(name)
                 table.add_rule(name, invalid_rule)
@@ -225,67 +203,12 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
             else:
                 ver = IPV6
                 table = ipt_mgr.ipv6['filter']
-            ichain_name = self._get_chain_name(
-                fwid, ver, constants.INGRESS_DIRECTION)
-            ochain_name = self._get_chain_name(
-                fwid, ver, constants.EGRESS_DIRECTION)
+            ichain_name = self._get_chain_name(fwid, ver, INGRESS_DIRECTION)
+            ochain_name = self._get_chain_name(fwid, ver, EGRESS_DIRECTION)
             table.add_rule(ichain_name, iptbl_rule)
             table.add_rule(ochain_name, iptbl_rule)
         self._enable_policy_chain(fwid, ipt_if_prefix)
 
-    def _find_changed_rules(self, pre_firewall, firewall):
-        """Find the rules changed between the current firewall
-        and the updating rule
-        """
-        changed_rules = []
-        fw_rules_list = firewall[f_const.FIREWALL_RULE_LIST]
-        pre_fw_rules_list = pre_firewall[f_const.FIREWALL_RULE_LIST]
-        for pre_fw_rule in pre_fw_rules_list:
-            for fw_rule in fw_rules_list:
-                if (pre_fw_rule.get('id') == fw_rule.get('id') and
-                    pre_fw_rule != fw_rule):
-                    changed_rules.append(pre_fw_rule)
-                    changed_rules.append(fw_rule)
-        return changed_rules
-
-    def _find_removed_rules(self, pre_firewall, firewall):
-        fw_rules_list = firewall[f_const.FIREWALL_RULE_LIST]
-        pre_fw_rules_list = pre_firewall[f_const.FIREWALL_RULE_LIST]
-        fw_rule_ids = [fw_rule.get('id') for fw_rule in fw_rules_list]
-        removed_rules = [pre_fw_rule for pre_fw_rule in
-            pre_fw_rules_list if pre_fw_rule.get('id') not in fw_rule_ids]
-        return removed_rules
-
-    def _find_new_rules(self, pre_firewall, firewall):
-        return self._find_removed_rules(firewall, pre_firewall)
-
-    def _remove_conntrack_new_firewall(self, agent_mode, apply_list, firewall):
-        """Remove conntrack when create new firewall"""
-        routers_list = list(set(apply_list))
-        for router_info in routers_list:
-            ipt_if_prefix_list = self._get_ipt_mgrs_with_if_prefix(
-                agent_mode, router_info)
-            for ipt_if_prefix in ipt_if_prefix_list:
-                ipt_mgr = ipt_if_prefix['ipt']
-                self.conntrack.flush_entries(ipt_mgr.namespace)
-
-    def _remove_conntrack_updated_firewall(self, agent_mode,
-                                           apply_list, pre_firewall, firewall):
-        """Remove conntrack when updated firewall"""
-        router_list = list(set(apply_list))
-        for router_info in router_list:
-            ipt_if_prefix_list = self._get_ipt_mgrs_with_if_prefix(
-                agent_mode, router_info)
-            for ipt_if_prefix in ipt_if_prefix_list:
-                ipt_mgr = ipt_if_prefix['ipt']
-                ch_rules = self._find_changed_rules(pre_firewall,
-                                                    firewall)
-                i_rules = self._find_new_rules(pre_firewall, firewall)
-                r_rules = self._find_removed_rules(pre_firewall, firewall)
-                removed_conntrack_rules_list = ch_rules + i_rules + r_rules
-                self.conntrack.delete_entries(removed_conntrack_rules_list,
-                                              ipt_mgr.namespace)
-
     def _remove_default_chains(self, nsid):
         """Remove fwaas default policy chain."""
         self._remove_chain_by_name(IPV4, FWAAS_DEFAULT_CHAIN, nsid)
@@ -294,8 +217,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
     def _remove_chains(self, fwid, ipt_mgr):
         """Remove fwaas policy chain."""
         for ver in [IPV4, IPV6]:
-            for direction in [constants.INGRESS_DIRECTION,
-                              constants.EGRESS_DIRECTION]:
+            for direction in [INGRESS_DIRECTION, EGRESS_DIRECTION]:
                 chain_name = self._get_chain_name(fwid, ver, direction)
                 self._remove_chain_by_name(ver, chain_name, ipt_mgr)
 
@@ -326,8 +248,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
 
         for (ver, tbl) in [(IPV4, ipt_mgr.ipv4['filter']),
                            (IPV6, ipt_mgr.ipv6['filter'])]:
-            for direction in [constants.INGRESS_DIRECTION,
-                              constants.EGRESS_DIRECTION]:
+            for direction in [INGRESS_DIRECTION, EGRESS_DIRECTION]:
                 chain_name = self._get_chain_name(fwid, ver, direction)
                 chain_name = iptables_manager.get_chain_name(chain_name)
                 if chain_name in tbl.chains:
@@ -349,33 +270,17 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
         self._add_rules_to_chain(ipt_mgr, IPV6, 'FORWARD', jump_rule)
 
     def _convert_fwaas_to_iptables_rule(self, rule):
-        action = FWAAS_TO_IPTABLE_ACTION_MAP[rule.get('action')]
-
-        # Output ordering is important here as it must exactly match what
-        # is returned by iptables-save.  If not we risk unnecessarily removing
-        # and readding rules.
-        args = []
-
-        args += self._protocol_arg(rule.get('protocol'))
-
-        args += self._ip_prefix_arg('s', rule.get('source_ip_address'))
-        args += self._ip_prefix_arg('d', rule.get('destination_ip_address'))
-
-        # iptables adds '-m protocol' when any source
-        # or destination port number is specified
-        if (rule.get('source_port') is not None or
-            rule.get('destination_port') is not None):
-            args += self._match_arg(rule.get('protocol'))
-
-        args += self._port_arg('sport',
+        action = 'ACCEPT' if rule.get('action') == 'allow' else 'DROP'
+        args = [self._protocol_arg(rule.get('protocol')),
+                self._port_arg('dport',
                                rule.get('protocol'),
-                               rule.get('source_port'))
-
-        args += self._port_arg('dport',
+                               rule.get('destination_port')),
+                self._port_arg('sport',
                                rule.get('protocol'),
-                               rule.get('destination_port'))
-
-        args += self._action_arg(action)
+                               rule.get('source_port')),
+                self._ip_prefix_arg('s', rule.get('source_ip_address')),
+                self._ip_prefix_arg('d', rule.get('destination_ip_address')),
+                self._action_arg(action)]
 
         iptables_rule = ' '.join(args)
         return iptables_rule
@@ -384,46 +289,24 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
         return '-m state --state INVALID -j DROP'
 
     def _allow_established_rule(self):
-        return '-m state --state RELATED,ESTABLISHED -j ACCEPT'
+        return '-m state --state ESTABLISHED,RELATED -j ACCEPT'
 
     def _action_arg(self, action):
-        if not action:
-            return []
-
-        args = ['-j', action]
-
-        return args
+        if action:
+            return '-j %s' % action
+        return ''
 
     def _protocol_arg(self, protocol):
-        if not protocol:
-            return []
-
-        args = ['-p', protocol]
-
-        return args
-
-    def _match_arg(self, protocol):
-        if not protocol:
-            return []
-
-        protocol_modules = {'udp': 'udp', 'tcp': 'tcp',
-                           'icmp': 'icmp', 'ipv6-icmp': 'icmp6'}
-        # iptables adds '-m protocol' when the port number is specified
-        args = ['-m', protocol_modules[protocol]]
-
-        return args
+        if protocol:
+            return '-p %s' % protocol
+        return ''
 
     def _port_arg(self, direction, protocol, port):
-        if protocol not in ['udp', 'tcp'] or port is None:
-            return []
-
-        args = ['--%s' % direction, '%s' % port]
-
-        return args
+        if not (protocol in ['udp', 'tcp'] and port):
+            return ''
+        return '--%s %s' % (direction, port)
 
     def _ip_prefix_arg(self, direction, ip_prefix):
-        if not(ip_prefix):
-            return []
-
-        args = ['-%s' % direction, '%s' % utils.ip_to_cidr(ip_prefix)]
-        return args
+        if ip_prefix:
+            return '-%s %s' % (direction, ip_prefix)
+        return ''
