@@ -14,17 +14,17 @@
 #
 
 from neutron.api.v2 import attributes as attr
+from neutron.common import constants as l3_const
+from neutron.common import log
 from neutron.common import rpc as n_rpc
 from neutron import context as neutron_context
+from neutron.i18n import _LW
 from neutron import manager
 from neutron.plugins.common import constants as const
-from neutron_lib import constants as l3_const
 from oslo_config import cfg
-from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 import oslo_messaging
 
-from neutron_fwaas._i18n import _LW
 from neutron_fwaas.db.cisco import cisco_fwaas_db as csrfw_db
 import neutron_fwaas.extensions
 from neutron_fwaas.extensions.cisco import csr_firewall_insertion as csr_ext
@@ -41,7 +41,7 @@ class FirewallCallbacks(object):
         super(FirewallCallbacks, self).__init__()
         self.plugin = plugin
 
-    @log_helpers.log_method_call
+    @log.log
     def set_firewall_status(self, context, firewall_id, status,
                             status_data=None, **kwargs):
         """Agent uses this to set a firewall's status."""
@@ -71,7 +71,7 @@ class FirewallCallbacks(object):
             else:
                 fw_db.status = const.ERROR
 
-    @log_helpers.log_method_call
+    @log.log
     def firewall_deleted(self, context, firewall_id, **kwargs):
         """Agent uses this to indicate firewall is deleted."""
         with context.session.begin(subtransactions=True):
@@ -80,13 +80,13 @@ class FirewallCallbacks(object):
             if fw_db.status in (const.PENDING_DELETE, const.ERROR):
                 self.plugin.delete_db_firewall_object(context, firewall_id)
                 return True
-            LOG.warning(_LW('Firewall %(fw)s unexpectedly deleted by agent, '
-                            'status was %(status)s'),
-                        {'fw': firewall_id, 'status': fw_db.status})
+            LOG.warn(_LW('Firewall %(fw)s unexpectedly deleted by agent, '
+                         'status was %(status)s'),
+                {'fw': firewall_id, 'status': fw_db.status})
             fw_db.status = const.ERROR
         return False
 
-    @log_helpers.log_method_call
+    @log.log
     def get_firewalls_for_tenant(self, context, **kwargs):
         """Agent uses this to get all firewalls and rules for a tenant."""
         fw_list = []
@@ -102,12 +102,12 @@ class FirewallCallbacks(object):
             fw_list.append(fw_with_rules)
         return fw_list
 
-    @log_helpers.log_method_call
+    @log.log
     def get_firewalls_for_tenant_without_rules(self, context, **kwargs):
         """Agent uses this to get all firewalls for a tenant."""
         return [fw for fw in self.plugin.get_firewalls(context)]
 
-    @log_helpers.log_method_call
+    @log.log
     def get_tenants_with_firewalls(self, context, **kwargs):
         """Agent uses this to get all tenants that have firewalls."""
         ctx = neutron_context.get_admin_context()
@@ -164,7 +164,7 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
 
         self.endpoints = [FirewallCallbacks(self)]
 
-        self.conn = n_rpc.create_connection()
+        self.conn = n_rpc.create_connection(new=True)
         self.conn.create_consumer(
             'CISCO_FW_PLUGIN', self.endpoints, fanout=False)
         self.conn.consume_in_threads()
@@ -190,7 +190,7 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
                 fw_with_rules)
             self.agent_rpc.update_firewall(context, fw_with_rules)
 
-    @log_helpers.log_method_call
+    @log.log
     def _validate_fw_port_and_get_router_id(self, context, tenant_id, port_id):
         # port validation with router plugin
         l3_plugin = manager.NeutronManager.get_service_plugins().get(
@@ -240,8 +240,10 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
                 return vendor_ext
         # TODO(sridar): we may need to raise an excp - check backlogging
 
-    @log_helpers.log_method_call
+    @log.log
     def create_firewall(self, context, firewall):
+        tenant_id = self._get_tenant_id_for_create(context,
+                                                   firewall['firewall'])
         port_id = firewall['firewall'].pop('port_id', None)
         direction = firewall['firewall'].pop('direction', None)
 
@@ -253,7 +255,7 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
             # TODO(sridar): add check to see if the new port-id does not have
             # any associated firewall.
             router_id = self._validate_fw_port_and_get_router_id(context,
-                firewall['firewall']['tenant_id'], port_id)
+                tenant_id, port_id)
 
         if direction == attr.ATTR_NOT_SPECIFIED:
             direction = None
@@ -280,9 +282,11 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
             self.agent_rpc.create_firewall(context, fw_with_rules)
         return fw
 
-    @log_helpers.log_method_call
+    @log.log
     def update_firewall(self, context, fwid, firewall):
         self._ensure_update_firewall(context, fwid)
+        tenant_id = self._get_tenant_id_for_create(context,
+                                                   firewall['firewall'])
         csrfw = self.lookup_firewall_csr_association(context, fwid)
 
         port_id = firewall['firewall'].pop('port_id', None)
@@ -291,7 +295,6 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
         _fw = {'id': fwid}
 
         if port_id:
-            tenant_id = self.get_firewall(context, fwid)['tenant_id']
             router_id = self._validate_fw_port_and_get_router_id(context,
                 tenant_id, port_id)
             if csrfw and csrfw['port_id']:
@@ -334,7 +337,7 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
             self.agent_rpc.update_firewall(context, fw_with_rules)
         return fw
 
-    @log_helpers.log_method_call
+    @log.log
     def delete_firewall(self, context, fwid):
         self._ensure_update_firewall(context, fwid)
 
@@ -360,7 +363,7 @@ class CSRFirewallPlugin(ref_fw_plugin.FirewallPlugin,
 
         self.agent_rpc.delete_firewall(context, fw_with_rules)
 
-    @log_helpers.log_method_call
+    @log.log
     def get_firewall(self, context, fwid, fields=None):
         res = super(ref_fw_plugin.FirewallPlugin, self).get_firewall(
                         context, fwid, fields)
