@@ -22,10 +22,10 @@ from neutron.agent.l3 import ha
 from neutron.agent.l3 import router_info
 from neutron.agent.linux import interface
 from neutron.common import config as base_config
+from neutron.common import constants as l3_constants
+from neutron.openstack.common import uuidutils
 from neutron_fwaas.services.firewall.agents.varmour import varmour_router
 from neutron_fwaas.tests import base
-from neutron_lib import constants as l3_constants
-from oslo_utils import uuidutils
 
 
 _uuid = uuidutils.generate_uuid
@@ -43,6 +43,7 @@ class TestVarmourRouter(base.BaseTestCase):
         self.conf.register_opts(ha.OPTS)
         agent_config.register_process_monitor_opts(self.conf)
         agent_config.register_interface_driver_opts_helper(self.conf)
+        agent_config.register_use_namespaces_opts_helper(self.conf)
         self.conf.register_opts(interface.OPTS)
         self.conf.set_override('interface_driver',
                                'neutron.agent.linux.interface.NullDriver')
@@ -78,7 +79,7 @@ class TestVarmourRouter(base.BaseTestCase):
         mock.patch('neutron.agent.l3.agent.L3PluginApi').start()
 
         self.looping_call_p = mock.patch(
-            'oslo_service.loopingcall.FixedIntervalLoopingCall')
+            'neutron.openstack.common.loopingcall.FixedIntervalLoopingCall')
         self.looping_call_p.start()
         self.ri_kwargs = {'agent_conf': self.conf,
                           'interface_driver': self.mock_driver}
@@ -199,4 +200,4 @@ class TestVarmourRouter(base.BaseTestCase):
         ri = self._prepare_router_data(enable_snat=True)
         router._router_added(ri.router['id'], ri.router)
         ri.router['enable_snat'] = False
-        self.assertFalse(ri.router['enable_snat'])
+        self.assertEqual(ri.router['enable_snat'], False)
