@@ -13,12 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import copy
-
 import mock
+from oslo_config import cfg
+
+from neutron.agent.common import config as a_cfg
 from neutron.tests import base
 from neutron.tests.unit.api.v2 import test_base as test_api_v2
-
 import neutron_fwaas.services.firewall.drivers.linux.iptables_fwaas as fwaas
 
 
@@ -29,21 +29,19 @@ FAKE_PROTOCOL = 'tcp'
 FAKE_SRC_PORT = 5000
 FAKE_DST_PORT = 22
 FAKE_FW_ID = 'fake-fw-uuid'
-FW_LEGACY = 'legacy'
 
 
 class IptablesFwaasTestCase(base.BaseTestCase):
     def setUp(self):
         super(IptablesFwaasTestCase, self).setUp()
-        self.conntrack_driver = mock.Mock()
-        self.conntrack_driver.initialize = mock.Mock()
-        self.conntrack_driver.delete_entries = mock.Mock()
-        self.conntrack_driver.flush_entries = mock.Mock()
+        cfg.CONF.register_opts(a_cfg.ROOT_HELPER_OPTS, 'AGENT')
+        self.utils_exec_p = mock.patch(
+            'neutron.agent.linux.utils.execute')
+        self.utils_exec = self.utils_exec_p.start()
         self.iptables_cls_p = mock.patch(
             'neutron.agent.linux.iptables_manager.IptablesManager')
         self.iptables_cls_p.start()
         self.firewall = fwaas.IptablesFwaasDriver()
-        self.firewall.conntrack = self.conntrack_driver
 
     def _fake_rules_v4(self, fwid, apply_list):
         rule_list = []
@@ -52,20 +50,12 @@ class IptablesFwaasTestCase(base.BaseTestCase):
                  'ip_version': 4,
                  'protocol': 'tcp',
                  'destination_port': '80',
-                 'source_ip_address': '10.24.4.2',
-                 'id': 'fake-fw-rule1'}
+                 'source_ip_address': '10.24.4.2'}
         rule2 = {'enabled': True,
                  'action': 'deny',
                  'ip_version': 4,
                  'protocol': 'tcp',
-                 'destination_port': '22',
-                 'id': 'fake-fw-rule2'}
-        rule3 = {'enabled': True,
-                 'action': 'reject',
-                 'ip_version': 4,
-                 'protocol': 'tcp',
-                 'destination_port': '23',
-                 'id': 'fake-fw-rule3'}
+                 'destination_port': '22'}
         ingress_chain = ('iv4%s' % fwid)[:11]
         egress_chain = ('ov4%s' % fwid)[:11]
         for router_info_inst in apply_list:
@@ -74,7 +64,6 @@ class IptablesFwaasTestCase(base.BaseTestCase):
             v4filter_inst.chains.append(egress_chain)
         rule_list.append(rule1)
         rule_list.append(rule2)
-        rule_list.append(rule3)
         return rule_list
 
     def _fake_firewall_no_rule(self):
@@ -86,13 +75,10 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         return fw_inst
 
     def _fake_firewall(self, rule_list):
-        _rule_list = copy.deepcopy(rule_list)
-        for rule in _rule_list:
-            rule['position'] = str(_rule_list.index(rule))
         fw_inst = {'id': FAKE_FW_ID,
                    'admin_state_up': True,
                    'tenant_id': 'tenant-uuid',
-                   'firewall_rule_list': _rule_list}
+                   'firewall_rule_list': rule_list}
         return fw_inst
 
     def _fake_firewall_with_admin_down(self, rule_list):
@@ -121,7 +107,7 @@ class IptablesFwaasTestCase(base.BaseTestCase):
             router_info_inst.iptables_manager = iptables_inst
             router_info_inst.snat_iptables_manager = iptables_inst
             if distributed_mode == 'dvr':
-                router_info_inst.rtr_fip_connect = True
+                router_info_inst.dist_fip_count = 1
             router_info_inst.router = router_inst
             apply_list.append(router_info_inst)
             router_count -= 1
@@ -143,10 +129,9 @@ class IptablesFwaasTestCase(base.BaseTestCase):
             distributed_mode = 'legacy'
         func(distributed_mode, apply_list, firewall)
         invalid_rule = '-m state --state INVALID -j DROP'
-        est_rule = '-m state --state RELATED,ESTABLISHED -j ACCEPT'
-        rule1 = '-p tcp -s 10.24.4.2/32 -m tcp --dport 80 -j ACCEPT'
-        rule2 = '-p tcp -m tcp --dport 22 -j DROP'
-        rule3 = '-p tcp -m tcp --dport 23 -j REJECT'
+        est_rule = '-m state --state ESTABLISHED,RELATED -j ACCEPT'
+        rule1 = '-p tcp --dport 80  -s 10.24.4.2  -j ACCEPT'
+        rule2 = '-p tcp --dport 22    -j DROP'
         ingress_chain = 'iv4%s' % firewall['id']
         egress_chain = 'ov4%s' % firewall['id']
         bname = fwaas.iptables_manager.binary_name
@@ -169,8 +154,6 @@ class IptablesFwaasTestCase(base.BaseTestCase):
                      mock.call.add_rule(egress_chain, rule1),
                      mock.call.add_rule(ingress_chain, rule2),
                      mock.call.add_rule(egress_chain, rule2),
-                     mock.call.add_rule(ingress_chain, rule3),
-                     mock.call.add_rule(egress_chain, rule3),
                      mock.call.add_rule('FORWARD',
                                         '-o %s -j %s' % (if_prefix,
                                         ipt_mgr_ichain)),
@@ -190,8 +173,9 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         firewall = self._fake_firewall_no_rule()
         self.firewall.create_firewall('legacy', apply_list, firewall)
         invalid_rule = '-m state --state INVALID -j DROP'
-        est_rule = '-m state --state RELATED,ESTABLISHED -j ACCEPT'
+        est_rule = '-m state --state ESTABLISHED,RELATED -j ACCEPT'
         bname = fwaas.iptables_manager.binary_name
+
         for ip_version in (4, 6):
             ingress_chain = ('iv%s%s' % (ip_version, firewall['id']))
             egress_chain = ('ov%s%s' % (ip_version, firewall['id']))
@@ -281,131 +265,3 @@ class IptablesFwaasTestCase(base.BaseTestCase):
     def test_update_firewall_with_rules_dvr(self):
         self._setup_firewall_with_rules(self.firewall.update_firewall,
             distributed=True, distributed_mode='dvr')
-
-    def test_remove_conntrack_new_firewall(self):
-        apply_list = self._fake_apply_list()
-        firewall = self._fake_firewall_no_rule()
-        self.firewall.create_firewall(FW_LEGACY, apply_list, firewall)
-        for router_info_inst in apply_list:
-            namespace = router_info_inst.iptables_manager.namespace
-            calls = [mock.call(namespace)]
-            self.conntrack_driver.flush_entries.assert_has_calls(calls)
-
-    def test_remove_conntrack_inserted_rule(self):
-        apply_list = self._fake_apply_list()
-        rule_list = self._fake_rules_v4(FAKE_FW_ID, apply_list)
-        firewall = self._fake_firewall(rule_list)
-        self.firewall.create_firewall(FW_LEGACY, apply_list, firewall)
-        self.firewall.pre_firewall = dict(firewall)
-        insert_rule = {'enabled': True,
-                 'action': 'deny',
-                 'ip_version': 4,
-                 'protocol': 'icmp',
-                 'id': 'fake-fw-rule'}
-        rule_list.insert(2, insert_rule)
-        firewall = self._fake_firewall(rule_list)
-        self.firewall.update_firewall(FW_LEGACY, apply_list, firewall)
-        rules_changed = [
-            {'destination_port': '23',
-             'position': '2',
-             'protocol': 'tcp',
-             'ip_version': 4,
-             'enabled': True,
-             'action': 'reject',
-             'id': 'fake-fw-rule3'},
-            {'destination_port': '23',
-             'position': '3',
-             'protocol': 'tcp',
-             'ip_version': 4,
-             'enabled': True,
-             'action': 'reject',
-             'id': 'fake-fw-rule3'}
-        ]
-        rules_inserted = [
-            {'id': 'fake-fw-rule',
-             'protocol': 'icmp',
-             'ip_version': 4,
-             'enabled': True,
-             'action': 'deny',
-             'position': '2'}
-        ]
-        for router_info_inst in apply_list:
-            namespace = router_info_inst.iptables_manager.namespace
-            self.conntrack_driver.delete_entries.assert_called_once_with(
-                rules_changed + rules_inserted, namespace
-            )
-
-    def test_remove_conntrack_removed_rule(self):
-        apply_list = self._fake_apply_list()
-        rule_list = self._fake_rules_v4(FAKE_FW_ID, apply_list)
-        firewall = self._fake_firewall(rule_list)
-        self.firewall.create_firewall(FW_LEGACY, apply_list, firewall)
-        self.firewall.pre_firewall = dict(firewall)
-        remove_rule = rule_list[1]
-        rule_list.remove(remove_rule)
-        firewall = self._fake_firewall(rule_list)
-        self.firewall.update_firewall(FW_LEGACY, apply_list, firewall)
-        rules_changed = [
-            {'destination_port': '23',
-             'position': '2',
-             'protocol': 'tcp',
-             'ip_version': 4,
-             'enabled': True,
-             'action': 'reject',
-             'id': 'fake-fw-rule3'},
-            {'destination_port': '23',
-             'position': '1',
-             'protocol': 'tcp',
-             'ip_version': 4,
-             'enabled': True,
-             'action': 'reject',
-             'id': 'fake-fw-rule3'}
-        ]
-        rules_removed = [
-            {'enabled': True,
-             'position': '1',
-             'protocol': 'tcp',
-             'id': 'fake-fw-rule2',
-             'ip_version': 4,
-             'action': 'deny',
-             'destination_port': '22'}
-        ]
-        for router_info_inst in apply_list:
-            namespace = router_info_inst.iptables_manager.namespace
-            self.conntrack_driver.delete_entries.assert_called_once_with(
-                rules_changed + rules_removed, namespace
-            )
-
-    def test_remove_conntrack_changed_rule(self):
-        apply_list = self._fake_apply_list()
-        rule_list = self._fake_rules_v4(FAKE_FW_ID, apply_list)
-        firewall = self._fake_firewall(rule_list)
-        self.firewall.create_firewall(FW_LEGACY, apply_list, firewall)
-        income_rule = {'enabled': True,
-                 'action': 'deny',
-                 'ip_version': 4,
-                 'protocol': 'tcp',
-                 'id': 'fake-fw-rule3'}
-        rule_list[2] = income_rule
-        firewall = self._fake_firewall(rule_list)
-        self.firewall.update_firewall(FW_LEGACY, apply_list, firewall)
-        rules_changed = [
-            {'id': 'fake-fw-rule3',
-             'enabled': True,
-             'action': 'reject',
-             'position': '2',
-             'destination_port': '23',
-             'ip_version': 4,
-             'protocol': 'tcp'},
-            {'position': '2',
-             'enabled': True,
-             'action': 'deny',
-             'id': 'fake-fw-rule3',
-             'ip_version': 4,
-             'protocol': 'tcp'}
-        ]
-        for router_info_inst in apply_list:
-            namespace = router_info_inst.iptables_manager.namespace
-            self.conntrack_driver.delete_entries.assert_called_once_with(
-                rules_changed, namespace
-            )
