@@ -22,10 +22,9 @@ from neutron.agent.l3 import ha
 from neutron.agent.l3 import router_info
 from neutron.agent.linux import interface
 from neutron.common import config as base_config
+from neutron.common import constants as l3_constants
+from neutron.openstack.common import uuidutils
 from neutron.tests import base
-from neutron_lib import constants as l3_constants
-from oslo_utils import uuidutils
-
 from neutron_fwaas.services.firewall.agents.varmour import varmour_router
 from neutron_fwaas.services.firewall.drivers.varmour import varmour_fwaas
 
@@ -44,6 +43,7 @@ class TestBasicRouterOperations(base.BaseTestCase):
         self.conf.register_opts(ha.OPTS)
         agent_config.register_process_monitor_opts(self.conf)
         agent_config.register_interface_driver_opts_helper(self.conf)
+        agent_config.register_use_namespaces_opts_helper(self.conf)
         self.conf.register_opts(interface.OPTS)
         self.conf.set_override('interface_driver',
                                'neutron.agent.linux.interface.NullDriver')
@@ -79,7 +79,7 @@ class TestBasicRouterOperations(base.BaseTestCase):
         mock.patch('neutron.agent.l3.agent.L3PluginApi').start()
 
         self.looping_call_p = mock.patch(
-            'oslo_service.loopingcall.FixedIntervalLoopingCall')
+            'neutron.openstack.common.loopingcall.FixedIntervalLoopingCall')
         self.looping_call_p.start()
 
         self.fake_agent_mode = None
