@@ -13,6 +13,7 @@
 #    under the License.
 #
 
+import contextlib
 import mock
 
 from neutron.api.v2 import attributes as attr
@@ -20,7 +21,6 @@ from neutron import context
 from neutron import manager
 from neutron.plugins.common import constants as const
 from neutron.tests.unit.extensions import test_l3 as test_l3_plugin
-import six
 
 import neutron_fwaas
 from neutron_fwaas.db.cisco import cisco_fwaas_db as csrfw_db
@@ -73,7 +73,7 @@ class CSR1kvFirewallTestCaseBase(test_db_firewall.FirewallPluginDbTestCase,
         cfg.CONF.set_default('allow_overlapping_ips', True)
         cfg.CONF.set_default('max_routes', 3)
         self.saved_attr_map = {}
-        for resource, attrs in six.iteritems(attr.RESOURCE_ATTRIBUTE_MAP):
+        for resource, attrs in attr.RESOURCE_ATTRIBUTE_MAP.iteritems():
             self.saved_attr_map[resource] = attrs.copy()
         if not core_plugin:
             core_plugin = CORE_PLUGIN_KLASS
@@ -109,20 +109,12 @@ class CSR1kvFirewallTestCaseBase(test_db_firewall.FirewallPluginDbTestCase,
         self.restore_attribute_map()
         super(CSR1kvFirewallTestCaseBase, self).tearDown()
 
-    def _create_firewall(self, fmt, name, description, firewall_policy_id=None,
+    def _create_firewall(self, fmt, name, description, firewall_policy_id,
                          admin_state_up=True, expected_res_status=None,
                          **kwargs):
         tenant_id = kwargs.get('tenant_id', self._tenant_id)
         port_id = kwargs.get('port_id')
         direction = kwargs.get('direction')
-        if firewall_policy_id is None:
-            res = self._create_firewall_policy(fmt, 'fwp',
-                                               description="firewall_policy",
-                                               shared=True,
-                                               firewall_rules=[],
-                                               audited=True)
-            firewall_policy = self.deserialize(fmt or self.fmt, res)
-            firewall_policy_id = firewall_policy["firewall_policy"]["id"]
         data = {'firewall': {'name': name,
                              'description': description,
                              'firewall_policy_id': firewall_policy_id,
@@ -183,8 +175,10 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
 
     def test_create_csr_firewall(self):
 
-        with self.router(tenant_id=self._tenant_id) as r,\
-                self.subnet() as s:
+        with contextlib.nested(
+            self.router(tenant_id=self._tenant_id),
+            self.subnet(),
+        ) as (r, s):
 
             body = self._router_interface_action(
                 'add',
@@ -220,8 +214,10 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
 
     def test_create_csr_firewall_only_port_id_specified(self):
 
-        with self.router(tenant_id=self._tenant_id) as r, \
-                self.subnet() as s:
+        with contextlib.nested(
+            self.router(tenant_id=self._tenant_id),
+            self.subnet(),
+        ) as (r, s):
 
             body = self._router_interface_action(
                 'add',
@@ -252,7 +248,7 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
 
             self.assertEqual('firewall_1', fw['firewall']['name'])
             self.assertEqual(port_id, csrfw['port_id'])
-            self.assertIsNone(csrfw['direction'])
+            self.assertEqual(None, csrfw['direction'])
 
     def test_create_csr_firewall_no_port_id_no_direction_specified(self):
 
@@ -268,12 +264,14 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
                 ctx.session.flush()
 
             self.assertEqual('firewall_1', fw['firewall']['name'])
-            self.assertIsNone(csrfw)
+            self.assertEqual(None, csrfw)
 
     def test_update_csr_firewall(self):
 
-        with self.router(tenant_id=self._tenant_id) as r, \
-                self.subnet() as s:
+        with contextlib.nested(
+            self.router(tenant_id=self._tenant_id),
+            self.subnet(),
+        ) as (r, s):
 
             body = self._router_interface_action(
                 'add',
@@ -327,9 +325,11 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
 
     def test_update_csr_firewall_port_id(self):
 
-        with self.router(tenant_id=self._tenant_id) as r, \
-                self.subnet() as s1, \
-                self.subnet(cidr='20.0.0.0/24') as s2:
+        with contextlib.nested(
+            self.router(tenant_id=self._tenant_id),
+            self.subnet(),
+            self.subnet(cidr='20.0.0.0/24'),
+        ) as (r, s1, s2):
 
             body = self._router_interface_action(
                 'add',
@@ -392,9 +392,10 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
 
     def test_delete_csr_firewall(self):
 
-        with self.router(tenant_id=self._tenant_id) as r, \
-                self.subnet() as s:
-
+        with contextlib.nested(
+            self.router(tenant_id=self._tenant_id),
+            self.subnet(),
+        ) as (r, s):
             body = self._router_interface_action(
                 'add',
                 r['router']['id'],
@@ -412,7 +413,7 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
                 ctx = context.get_admin_context()
                 csrfw = self.lookup_firewall_csr_association(ctx,
                     fw_id)
-                self.assertIsNotNone(csrfw)
+                self.assertNotEqual(None, csrfw)
                 req = self.new_delete_request('firewalls', fw_id)
                 req.get_response(self.ext_api)
                 with ctx.session.begin(subtransactions=True):
@@ -422,7 +423,7 @@ class TestCiscoFirewallPlugin(CSR1kvFirewallTestCaseBase,
                 self.callbacks.firewall_deleted(ctx, fw_id)
                 csrfw = self.lookup_firewall_csr_association(ctx,
                     fw_id)
-                self.assertIsNone(csrfw)
+                self.assertEqual(None, csrfw)
             self._router_interface_action(
                 'remove',
                 r['router']['id'],
